@@ -1,6 +1,29 @@
-
-
-
+#' Shiny GUI for CLME
+#'
+#' @description Opens a graphical user interface to run \pkg{CLME}, built from the \pkg{shiny} package.
+#'
+#' @rdname shiny_clme
+#'
+#' @param input  input from GUI.
+#' @param output output to GUI.
+#' 
+#' @details 
+#' Currently the GUI does not allow specification of custom orders for the alternative hypothesis. Future versions may enable this capability.
+#' The data should be a CSV or table-delimited file with the first row being a header. Variables are identified using their column letter or number (e.g., 1 or A). Separate multiple variables with a comma (e.g., 1,2,4 or A,B,D), or select a range of variables with a dash (e.g., 1-4 or A-D). Set to 'None' (default) to indicate no covariates or random effects.
+#' If group levels for the constrained effect are character, they may not be read in the proper order. An extra column may contain the ordered group levels (it may therefore have different length than the rest of the dataset).
+#' 
+#' @note
+#' This function is primarily designed to be called by \code{\link{clme}}. 
+#' 
+#' By default, homogeneous variances are assumed for the residuals and (if included) random effects. Heterogeneity can be induced using the arguments \code{Nks} and \code{Qs}, which refer to the vectors \eqn{ (n_{1}, n_{2}, \ldots, n_{k}) }{(n1, n2 ,... , nk)} and \eqn{ (c_{1}, c_{2}, \ldots, c_{q}) }{(c1, c2 ,... , cq)}, respectively. See \code{\link{clme_em}} for further explanation of these values.
+#' 
+#' 
+#' @examples
+#' \dontrun{ shiny_clme() }
+#' 
+#' @import shiny
+#' @export
+#' 
 shiny_clme <- function(){
   library("CLME")
   runApp(
@@ -21,6 +44,12 @@ shiny_clme <- function(){
 ##############################################################################
 #  shinyUI(bootstrapPage())
 
+#'
+#' @rdname shiny_clme
+#' 
+#' @export
+#' 
+
 shinyUI_clme <- fluidPage( 
   titlePanel("Constrained Linear Mixed Effects"),
   sidebarLayout(
@@ -30,11 +59,11 @@ shinyUI_clme <- fluidPage(
       ## Data input
       ##
       fileInput('file1', 'Data (choose CSV file)',
-                accept=c('text/csv', 'text/comma-separated-values,text/plain', '.csv')
+                accept=c('text/csv', 'text/comma-separated-values,text/plain', '.csv', '.xlsx')
       ),
       selectInput(inputId = "dlmt",
                   label = "Delimiter:",
-                  choices=c("Comma", "Tab") 
+                  choices=c("Comma-delimited", "Tab-delimited", "xlsx") 
       ), 
       ##
       ## Main controls
@@ -55,6 +84,10 @@ shinyUI_clme <- fluidPage(
                          label = "Order (select at least one):",
                          choices=c("Simple","Umbrella","Tree")
       ),
+      checkboxInput(inputId = "decreasing",
+                    label = "Decreasing (inverted) order:",
+                    value=FALSE
+      ),
       helpText("Identify columns of data"),
       helpText("Use column letters or numbers"),
       helpText("e.g., 1-3 or A-C or a list: 1,4,6"),
@@ -66,10 +99,7 @@ shinyUI_clme <- fluidPage(
                    label = "Column(s) of Covariates:", value="None"),
       textInput(inputId = "q",
                    label = "Column(s) of random effects:", value="None"),
-      checkboxInput(inputId = "decreasing",
-                    label = "Decreasing order (inverted):",
-                    value=FALSE
-      ),
+
       numericInput(inputId = "nsim",
                    label = "Number of Bootstraps:",
                    min=0 , max=50000 , value=1000
@@ -111,6 +141,12 @@ shinyUI_clme <- fluidPage(
                                    min=0 , max=0.15 , value=0.05 , step=0.01
                        )
       ),
+      conditionalPanel(condition = "input.outcheck",
+                       checkboxInput(inputId = "makeFactor",
+                                   label = "Force constrained effect to be factor",
+                                   value=FALSE
+                       )
+      ),
       #br(),
       #conditionalPanel(condition = "input.outcheck",
       #                 helpText("Number of decimal places for:")
@@ -187,6 +223,9 @@ shinyUI_clme <- fluidPage(
         ), 
         tabPanel("Model Plot"   ,
                  plotOutput(outputId = "fig1", height = "650px")
+        ),
+        tabPanel("Model Data"   ,
+                 dataTableOutput(outputId = "datatbl")
         )
       )
     )
@@ -202,14 +241,19 @@ shinyUI_clme <- fluidPage(
 ##
 ##############################################################################
 
-
+#'
+#' @rdname shiny_clme
+#' 
+#' 
+#' @import graphics
+#' @importFrom openxlsx read.xlsx
+#' @importFrom utils read.csv
+#' 
+#' @export
+#' 
 shinyServer_clme <- function(input, output) {
-  
-  
-  #library("CLME")
-  
-  
-  clme.out <- reactive({
+    
+  clme_out <- reactive({
     
     compute1 <- input$compute1
     
@@ -222,19 +266,29 @@ shinyServer_clme <- function(input, output) {
         dlmt    <- input$dlmt
         #data1   <- as.matrix( read.csv( file=paste(file1) ) )
         
-        if( dlmt=="Comma"){
+        if( dlmt=="Comma-delimited"){
           data1   <- read.csv( file=paste(file1) )
         }
-        if( dlmt=="Tab" ){
+        if( dlmt=="Tab-delimited" ){
           data1   <- read.csv( file=paste(file1) , sep="\t")
         }
-        
+        if( dlmt=="xlsx"){
+          data1   <- read.xlsx( xlsxFile=paste(file1), colNames=TRUE )
+        }
+                
         yy      <- input$yy
         p1      <- input$p1
         p2      <- input$p2
         q       <- input$q
         nsim    <- input$nsim
         
+        alpha      <- 0.05        
+        makeFactor <- FALSE
+        if( input$outcheck==TRUE ){
+          makeFactor <- input$makeFactor
+          alpha      <- input$alpha
+        }
+        
         if( p2 == '' | p2==0 ){ p2 <- "None" }
         if(  q == '' |  q==0 ){  q <- "None" }
         
@@ -329,7 +383,8 @@ shinyServer_clme <- function(input, output) {
         }
         
         ncon   <- length(idx_x1)
-                
+        
+        
         ## Create the formula
         
         ## Reorder the levels of X1 if need be
@@ -341,6 +396,8 @@ shinyServer_clme <- function(input, output) {
             xlevels <- xlevels[ -which(xlevels=="")]  
           }
           data1[,idx_x1] <- factor( data1[,idx_x1], levels=xlevels )
+        } else if( ncon==1 & makeFactor==TRUE ){
+          data1[,idx_x1] <- factor( data1[,idx_x1] )
         }
         
         ## Build the formula
@@ -363,7 +420,8 @@ shinyServer_clme <- function(input, output) {
           uf      <-  paste( "(1|", uun , ")", collapse=" + " )
           formula <- formula( paste( yn, "~", x1f, "+", x2f, "+", uf ) )
         }
-              
+                
+        
         ## Input control arguments
         tsf <- lrt.stat
         if( input$tsfunc=="Williams" ){
@@ -380,7 +438,7 @@ shinyServer_clme <- function(input, output) {
         gfix <- NULL
         if( input$varssq ){
           idx_grp <- parse_idx(input$gfix)
-          gfix <- data[,idx_grp]
+          gfix <- data1[,idx_grp]
         }
         
         emiter <- 500
@@ -396,27 +454,30 @@ shinyServer_clme <- function(input, output) {
           mqeps  <- input$mqeps
           seedvl <- input$ranseed
         }
-              
+        
         ## Run the model
         data2 <- as.data.frame( data1 )
         
-        
-        if( ncon==1 & input$xlevel1 ){
-          clme.results <- clme(
-            formula=formula, data=data2, gfix=gfix, constraints=constraints, nsim=nsim, 
-            verbose = c(FALSE, FALSE, FALSE), tsf=tsf, tsf.ind = w.stat.ind,
-            mySolver=mySolver, seed=seedvl, ncon=ncon, levels=list(idx_x1, xlevels), 
-            em.eps=emeps, em.iter=emiter, mq.eps=mqeps, mq.iter=mqiter
-          )
-        } else{
-          clme.results <- clme(
-            formula=formula, data=data2, gfix=gfix, constraints=constraints, nsim=nsim, 
-            verbose = c(FALSE, FALSE, FALSE), tsf=tsf, tsf.ind = w.stat.ind,
-            mySolver=mySolver, seed=seedvl, ncon=ncon,
-            em.eps=emeps, em.iter=emiter, mq.eps=mqeps, mq.iter=mqiter
-          )
-        }
-        
+        withProgress(message = 'Status:', value = 1, {
+          setProgress(1/2, detail = paste("Computing"))
+          
+          if( ncon==1 & input$xlevel1 ){
+            clme.results <- summary( clme(
+              formula=formula, data=data2, gfix=gfix, constraints=constraints, 
+              verbose = c(FALSE, FALSE, FALSE), tsf=tsf, tsf.ind = w.stat.ind,
+              mySolver=mySolver, ncon=ncon, levels=list(idx_x1, xlevels), 
+              em.eps=emeps, em.iter=emiter, mq.eps=mqeps, mq.iter=mqiter
+            ), alpha=alpha, seed=seedvl, nsim=nsim )
+          } else{
+            clme.results <- summary( clme(
+              formula=formula, data=data2, gfix=gfix, constraints=constraints, 
+              verbose = c(FALSE, FALSE, FALSE), tsf=tsf, tsf.ind = w.stat.ind,
+              mySolver=mySolver, ncon=ncon,
+              em.eps=emeps, em.iter=emiter, mq.eps=mqeps, mq.iter=mqiter
+            ), alpha=alpha, seed=seedvl, nsim=nsim )
+          }
+          
+        })
         
         clme.results
         
@@ -430,119 +491,117 @@ shinyServer_clme <- function(input, output) {
   ## Boxplot of the data
   ##
   output$fig0 <- renderPlot({
-    
-    clme_out <- clme.out()
-    dframe   <- clme_out$dframe
-    
-    if( is.factor( dframe[,2] ) ){
-      if( length(levels(dframe[,2]))==clme_out$P1 ){        
-        boxplot( dframe[,1] ~ dframe[,2],
-                 xlab=colnames(dframe)[2], ylab=colnames(dframe)[1]  )
-      }
-      
+    clme_out <- clme_out()
+    if( length(clme_out)>1 ){
+      dframe   <- clme_out$dframe
+      if( is.factor( dframe[,2] ) ){
+        if( length(levels(dframe[,2]))==clme_out$P1 ){        
+          boxplot( dframe[,1] ~ dframe[,2],
+                   xlab=colnames(dframe)[2], ylab=colnames(dframe)[1]  )
+        }
         #if( (ncol(dframe)-1) >= clme_out$P1 ){
         #  xx1 <- apply( dframe[,2:(clme_out$P1+1)], 2, FUN=function(x){ (max(x)==1)*(min(x)==0) }  )
         #  
         #}
-      
+      } else{
+        xx <- yy <- c(0,1)  
+        plot( xx, yy, xlab="", ylab="", xaxt='n', yaxt='n', main="", col=0 )
+        legend( "top", inset=0.45, box.lwd=0,
+                legend="Cannot detect appropriate plot type.\n Try making constrained variable a factor.")
+      }
     } else{
-      xx <- yy <- c(0,1)  
-      plot( xx, yy, xlab="", ylab="", xaxt='n', yaxt='n', main="", col=0 )
-      legend( "top", inset=0.45, box.lwd=0,
-              legend="Cannot detect appropriate plot type.\n Try making constrained variable a factor.")
+      plot( 1:5 , 1:5 , col="white", xaxt='n', yaxt='n', xlab="", ylab="", frame.plot=FALSE )
     }
-          
   })
   
   
   output$sum_table <- renderTable({
-    clme_out <- clme.out()
-    dframe   <- clme_out$dframe
+    clme_out <- clme_out()
     
-    funq1 <- function(x) quantile(x, 0.25 )
-    funq3 <- function(x) quantile(x, 0.75 )
-    
-    #if( is.factor( dframe[,2] )){
-    #  if( length(levels(dframe[,2]))==clme_out$P1 ){    
-        
-        
-        xbar <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="mean")
-        ndat <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="length")
-        stdv <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="sd")
-        minx <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="min")
-        maxx <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="max")
-        medn <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="median")
-        qrt1 <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN=funq1)
-        qrt3 <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN=funq3)
-        
-        tbl1 <- cbind( ndat, xbar[,2], stdv[,2], minx[,2], qrt1[,2], medn[,2], qrt3[,2], maxx[,2] )
-        colnames(tbl1) <- c("Groups", "N", "Mean", "Std", "Min", "Q1", "Med", "Q3", "Max")
-        
-        format(tbl1, digits=3)
-        
-    #  }
+    if( length(clme_out)>1 ){
+      dframe   <- clme_out$dframe
       
-      #if( (ncol(dframe)-1) >= clme_out$P1 ){
-      #  xx1 <- apply( dframe[,2:(clme_out$P1+1)], 2, FUN=function(x){ (max(x)==1)*(min(x)==0) }  )
-      #  
-      #}
+      funq1 <- function(x) quantile(x, 0.25 )
+      funq3 <- function(x) quantile(x, 0.75 )
       
+      xbar <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="mean")
+      ndat <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="length")
+      stdv <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="sd")
+      minx <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="min")
+      maxx <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="max")
+      medn <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN="median")
+      qrt1 <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN=funq1)
+      qrt3 <- aggregate( dframe[,1] , by=list(dframe[,2]), FUN=funq3)
       
-    #} else{
-    #  tbl1 <- data.frame(
-    #    Variable = colnames(dframe)[1],
-    #    N        = nrow(dframe),
-    #    Std      = sd(dframe[,1]),
-    #    Min      = min(dframe[,1]),
-    #    Q1       = funq1(dframe[,1])[1],
-    #    Med      = median(dframe[,1]),
-    #    Q3       = funq3(dframe[,1])[1],
-    #    Max      = max(dframe[,1])
-    #    )
-    #  rownames(tbl1) <- NULL  
-    #  format(tbl1, digits=3)
-    #}
+      tbl1 <- cbind( ndat, xbar[,2], stdv[,2], minx[,2], qrt1[,2], medn[,2], qrt3[,2], maxx[,2] )
+      colnames(tbl1) <- c("Groups", "N", "Mean", "Std", "Min", "Q1", "Med", "Q3", "Max")
+      
+      format(tbl1, digits=3)
+      
+    } else{
+      tbl1 <- matrix( "No results yet" , ncol=1, nrow=1)
+      format(tbl1, digits=3)      
+    }
+
   })
   
   
-  
-  
-  
   ##
   ## Summarize the model
   ##
   output$fig1 <- renderPlot({
-    compute2 <- input$compute2
-    if( compute2 > 0 ){
-      isolate({
-        ciwd  <- FALSE
-        alpha <- 0.05
-        if( input$outcheck==TRUE ){
-          ciwd  <- input$plotci
-          alpha <- input$alpha
-        }
-        plot( clme.out() , ci=ciwd , alpha=alpha)
-      })
+    
+    clme_out <- clme_out()
+    if( length(clme_out)>1 ){
+      compute2 <- input$compute2
+      if( compute2 > 0 ){
+        
+        isolate({
+          ciwd  <- FALSE
+          alpha <- 0.05
+          if( input$outcheck==TRUE ){
+            ciwd  <- input$plotci
+            alpha <- input$alpha
+          }
+          plot( clme_out , ci=ciwd , alpha=alpha)
+        })
+      }      
+    } else{
+      plot( 1:5 , 1:5 , col="white", xaxt='n', yaxt='n', xlab="", ylab="", frame.plot=FALSE )
     }
+
   })
   
   output$summary <- renderPrint({
-    compute3 <- input$compute3
-    if( compute3 > 0 ){
-      isolate({
-        alpha <- 0.05        
-        if( input$outcheck==TRUE ){
-          alpha <- input$alpha
-        }
-        summary( clme.out() , alpha=alpha )
-      })
+    
+    clme_out <- clme_out()
+    if( length(clme_out)>1 ){
+      compute3 <- input$compute3
+      if( compute3 > 0 ){
+        isolate({
+          clme_out
+        })
+      }
+    } else{
+      print( "Model has not yet been run."  )
+    }
+    
+  })
+  
+
+  output$datatbl <-  renderDataTable({
+    clme_out <- clme_out()
+    if( length(clme_out) > 1 ){
+      clme_out$dframe
     }
   })
   
   
-  ## Put all the code to run CLME inside this
-  ## That probably won't work; too much formatting and conditional
-  # output$data <- renderTable({ clme.out() })
+  ## To check the output of various things when I'm updating
+  #output$etc <-  renderPrint({
+  #  print( clme_out() )
+  #})
+  
   
 }
 
